#include <stdlib.h>
#include "SDL.h"

const int COLORS = 16;
const int CHARS = 256;
const int CHAR_WIDTH = 8;
const int CHAR_HEIGHT = 14;

class SDLConsole {
public:
	// Structures
	struct Color {
		Uint8 Red;
		Uint8 Green;
		Uint8 Blue;
	};

	struct Character {
		Uint8 Row[CHAR_HEIGHT];
	};

	// Enumerations
	enum C_PaletteFormats {
		PALETTE_MEGAZEUX
	};

	enum C_CharSetFormats {
		CHARSET_MEGAZEUX
	};

	// Methods
	SDL_Surface CreateScreen(Uint16 width, Uint16 height);
	void Plot(Uint8 chr, Uint8 pal, Uint16 x, Uint16 y);
	void Plot(Uint8 chr, Uint8 pal, Uint16 x, Uint16 y, bool update);
	void Clear();
	void ClearRegion(Uint8 chr, Uint8 pal, Uint16 X1, Uint16 Y1, Uint16 X2, Uint16 Y2);
	void UpdateScreen();
	void Update(Uint16 X, Uint16 Y);
	void UpdateRegion(Uint16 X1, Uint16 Y1, Uint16 X2, Uint16 Y2);

	Uint8 GetCharacter(Uint16 X, Uint16 Y);
	Uint8 GetPalette(Uint16 X, Uint16 Y);
	void SetCharacter(Uint8 chr, Uint16 X, Uint16 Y);
	void SetPalette(Uint8 pal, Uint16 X, Uint16 Y);
	Uint8 **GetCharMap();
	Uint8 **GetColorMap();
	void SetCharMap(Uint8 **newCharMap);
	void SetColorMap(Uint8 **newColorMap);

	void ChangeCharacter(Uint8 index, Character newCharacter);
	void ChangePalette(Uint8 index, Uint8 red, Uint8 green, Uint8 blue);
	void ChangePalette(Uint8 index, Color color);

	bool LoadPalette(char *filename, C_PaletteFormats format);
	bool LoadCharset(char *filename, C_CharSetFormats format);
	
private:
	// Methods
	void PlotPixel(Uint16 x, Uint16 y, Uint8 R, Uint8 G, Uint8 B);
	void UpdateChar(Uint16 x, Uint16 y);

	bool LoadPaletteMegaZeux(char *filename);
	bool LoadCharsetMegaZeux(char *filename);

	// Variables
	SDL_Surface *screen;
	Uint16 screenWidth, screenHeight;
	Uint16 screenPxWidth, screenPxHeight;
	Uint8 **charMap;
	Uint8 **colorMap;
	Character charset[CHARS];
	Color palette[COLORS];
};