#include "SDLConsole.h"

void SDLConsole::UpdateChar(Uint16 X, Uint16 Y) {
	// get char/pal entry for the given position
	Character curChar = charset[charMap[X][Y]];
	Uint8 curCol = colorMap[X][Y];
	Uint8 fbColIndex = curCol << 4;
	fbColIndex >>= 4;
	Color bgCol = palette[curCol >> 4];
	Color fgCol = palette[fbColIndex];

	// calculate starting plot position
	Uint16 startX = X * CHAR_WIDTH;
	Uint16 startY = Y * CHAR_HEIGHT;

	for(Uint8 y = 0; y < CHAR_HEIGHT; y++)
		for(Uint8 x = 0; x < CHAR_WIDTH; x++)
			// determine whether to use background or foreground color for the current pixel
			if((curChar.Row[y] & (1 << (CHAR_WIDTH - x - 1))) > 0)
				PlotPixel(startX + x, startY + y, fgCol.Red, fgCol.Green, fgCol.Blue);
			else
				PlotPixel(startX + x, startY + y, bgCol.Red, bgCol.Green, bgCol.Blue);
}

void SDLConsole::UpdateScreen() {
	SDL_LockSurface(screen);

	for(Uint16 x = 0; x < screenWidth; x++)
		for(Uint16 y = 0; y < screenHeight; y++)
			UpdateChar(x, y);

	SDL_UnlockSurface(screen);
	SDL_UpdateRect(screen, 0, 0, 0, 0);
}

void SDLConsole::Update(Uint16 X, Uint16 Y) {
	SDL_LockSurface(screen);
	UpdateChar(X, Y);
	SDL_UnlockSurface(screen);
	SDL_UpdateRect(screen, X * CHAR_WIDTH, Y * CHAR_HEIGHT, CHAR_WIDTH, CHAR_HEIGHT);
}

void SDLConsole::UpdateRegion(Uint16 X1, Uint16 Y1, Uint16 X2, Uint16 Y2) {
	SDL_LockSurface(screen);

	for(Uint16 x = X1; x < (X2+1); x++)
		for(Uint16 y = Y1; y < (Y2+1); y++)
			UpdateChar(x, y);

	SDL_UnlockSurface(screen);
	SDL_UpdateRect(screen, X1 * CHAR_WIDTH, Y1 * CHAR_HEIGHT, ((X2 + 1) * CHAR_WIDTH) - (X1 * CHAR_WIDTH), ((Y2 + 1) * CHAR_HEIGHT) - (Y1 * CHAR_HEIGHT));
}