#include <stdlib.h>
#include "SDLConsole.h"

#if defined(_MSC_VER)
#	include "SDL.h"
#else
#	include "SDL/SDL.h"
#endif

SDL_Surface *screen;

const int SCREEN_WIDTH = 80;
const int SCREEN_HEIGHT = 25;

int main(int argc, char *argv[]) {
	SDLConsole console;

	// attempt to initialize SDL's video subsystem
	if(SDL_Init(SDL_INIT_VIDEO) < 0) {
		fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}

	// register quit event
	atexit(SDL_Quit);

	// attempt to create window
	screen = &console.CreateScreen(SCREEN_WIDTH, SCREEN_HEIGHT);

	// exit on error
	if(screen == NULL) {
		fprintf(stderr, "Unable to create rendering window: %s\n", SDL_GetError());
		exit(1);
	}

	SDL_WM_SetCaption("Console Test", NULL);

	// attempt to load palette and character set
	if(!console.LoadCharset("ascii.chr", console.CHARSET_MEGAZEUX)) {
		fprintf(stderr, "Unable to load character set\n");
		exit(2);
	}

	if(!console.LoadPalette("cga.pal", console.PALETTE_MEGAZEUX)) {
		fprintf(stderr, "Unable to load palette\n");
		exit(3);
	}
	
	// plot a few characters
	console.Plot(1, 0x0f, 5, 4);
	console.Plot(8, 0x0c, 10, 10);

	// zomg yellow borders
	for(unsigned int x = 0; x < SCREEN_WIDTH; x++) {
		console.Plot(178, 0x0e, x, 0);
		console.Plot(178, 0x0e, x, SCREEN_HEIGHT - 1);
	}

	for(unsigned int y = 0; y < SCREEN_HEIGHT; y++) {
		console.Plot(178, 0x0e, 0, y);
		console.Plot(178, 0x0e, SCREEN_WIDTH - 1, y);
	}

	// update screen
	console.UpdateScreen();

	// player position :o
	int pX = 40;
	int pY = 12;
	bool leftDown = false, rightDown = false, upDown = false, downDown = false;

	// fill part of the screen with stuff
	console.ClearRegion(0xb0, 0x2A, 60, 16, 78, 23);
	console.UpdateRegion(60, 16, 78, 23);

	// main loop
	while(1) {
		// poll for and handle events
		SDL_Event event;

		int oldX = pX;
		int oldY = pY;

		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				if(event.key.keysym.sym == SDLK_LEFT) leftDown = true;
				if(event.key.keysym.sym == SDLK_RIGHT) rightDown = true;
				if(event.key.keysym.sym == SDLK_UP) upDown = true;
				if(event.key.keysym.sym == SDLK_DOWN) downDown = true;
				break;
			case SDL_KEYUP:
				if(event.key.keysym.sym == SDLK_LEFT) leftDown = false;
				if(event.key.keysym.sym == SDLK_RIGHT) rightDown = false;
				if(event.key.keysym.sym == SDLK_UP) upDown = false;
				if(event.key.keysym.sym == SDLK_DOWN) downDown = false;

				// quit if escape is pressed
				if(event.key.keysym.sym == SDLK_ESCAPE) return 0;
				break;
			case SDL_QUIT:
				return 0;
			}
		}

		if(leftDown) pX--;
		if(rightDown) pX++;
		if(upDown) pY--;
		if(downDown) pY++;

		if(pX > 79) pX = 79;
		if(pX < 0) pX = 0;
		if(pY > 24) pY = 24;
		if(pY < 0) pY = 0;

		// update and plot player position if he/she moved
		console.Plot(0, 0, oldX, oldY);
		console.Plot(2, 0x1f, pX, pY);
		console.Update(oldX, oldY);
		console.Update(pX, pY);

		// wait for a mo'
		SDL_Delay(75);
	}

	return 0;
}